/*      beos.c
 *
 * BeOS(BAudioPlayer Sound Device
 *
 * $Id: beos v.0.1 alpha by johnkenneth@gmx.net
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of MIDAS Digital Audio System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Digital Audio System license, "license.txt". By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/

#include <SoundPlayer.h>

//int16 *tmp_buffer;

//#include "beos.h"



#include "lang.h"
#include "mtypes.h"
#include "errors.h"
#include "mglobals.h"
#include "sdevice.h"
#include "mmem.h"
#include "dsm.h"

#include "printf.h"
#include "malloc.h"

#define DEBUG_CODE(x)

RCSID(char const *beos_rcsid = "$Id: beos v.0.1 alpha by johnkenneth@gmx.net";)

static    int format_stereo;
static    uchar *audioBuffer;
static    int audioBufferSize;


#define BEOSVERSION 0.10
#define BEOSVERSTR "0.10"


/* Sound Device information */

    /* Sound Card names: */
static char     *beosCardName = "BeOS Audio Mixer";


/* Critical section object to prevent thread synchronization problems: */
//static CRITICAL_SECTION deviceUsed;

/* Sound Device internal static variables */
static unsigned mixRate, outputMode, mixMode;
static unsigned bufferLen;
static unsigned mixElemSize;
static unsigned bufferLeaveSpace;



int playing=0;
    unsigned    numElems;


static unsigned updateMix;              /* number of elements to mix between
                                           two updates */
static unsigned mixLeft;                /* number of elements to mix before
                                           next update */
static unsigned writePos;               /* write position */
static unsigned readPos;                /* read position */
/* Sound Device internal static variables from OSS*/
static unsigned spaceLeft;              /* number of bytes of space to write */

#ifdef DUMPBUFFER
static FILE     *buff;
#endif

//static LPDIRECTSOUND ds;                /* the DirectSound object */
//static LPDIRECTSOUNDBUFFER buf;         /* the buffer we will be using */
//static HWND         hwnd;               /* the HWND for DirectSound */

//static HMODULE dsModule = NULL;         /* dsound.dll module handle */

//static HRESULT (WINAPI *dsDirectSoundCreate)(const GUID *lpGUID,
//                                             LPDIRECTSOUND *ppDS,
//                                             IUnknown FAR *pUnkOuter) = NULL;

//static int      emulation;              /* are we running in emulation? */
//static int      primary;                /* primary buffer mode? */

//static int      suspended;              /* is playback suspended? */


/****************************************************************************\
*       enum dsFunctIDs
*       -----------------
* Description:  ID numbers for DirectSound Sound Device functions
\****************************************************************************/

enum beosFunctIDs
{
    ID_beosDetect = ID_beos,
    ID_beosInit,
    ID_beosClose,
    ID_beosGetMode,
    ID_beosSetUpdRate,
    ID_beosStartPlay,
    ID_beosPlay

};



/* Local prototypes: */
int CALLING beosSetUpdRate(unsigned updRate);

/*
typedef struct cookie_record {
float value;
float direction;
} cookie_record;

*/

void Be_Mix( void *thecookie, void *buffer, size_t size,
		const media_raw_audio_format &format )
{
//	if(!Player_Active()) Player_Stop();


//cookie_record *cookie = (cookie_record *) thecookie;
  	
	switch( format.format )
	{
		case media_raw_audio_format::B_AUDIO_UCHAR:
			{

				//	int8 *buf = (int8 *)buffer;

    int         error;
//    unsigned    numElems;


  	
    /* Calculate number of mixing elements left: */
    numElems = audioBufferSize / mixElemSize;    

	if (playing) {

    /* Mix the data: */
    if ( (error = dsmMixData(numElems, (int8 *)buffer)) != OK );
//            PASSERROR(ID_beosPlay)
 

//					playing=0;
	
	}


//cookie->value += cookie->direction*(1.0/64.0);



			}
			break;


		default: break;
					
	}		
}


/****************************************************************************
** Actual Driver
*****************************************************************************/
BSoundPlayer			*soco_bsbp = NULL;
media_raw_audio_format		soco_mf;




/****************************************************************************\
*
* Function:     int dsDetect(int *result)
*
* Description:  Detects a DirectSound Sound Device
*
* Input:        int *result             pointer to detection result
*
* Returns:      MIDAS error code. Detection result (1 if detected, 0 if not)
*               is written to *result.
*
\****************************************************************************/

int CALLING beosDetect(int *result)
{
    /* If the output device can be opened, we have sound: */
  	*result = 1;


    return OK;
}





/****************************************************************************\
*
* Function:     int dsInit(unsigned mixRate, unsigned mode)
*
* Description:  Initializes DirectSound Sound Device
*
* Input:        unsigned mixRate        mixing rate in Hz
*               unsigned mode           output mode
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING beosInit(unsigned _mixRate, unsigned mode)
{
    int         error;
    int         mixMode;
    int         *modetag;
   mixRate = _mixRate;



	printf("ok here mixrate: %i \n",mixRate);




   /* Determine the actual output mode: */
    if ( mode & sdMono ) {
		format_stereo = 1;
	}
    else {
		format_stereo = 2;
	}


    /* Check what format we got and use it */   
    if (format_stereo == 2)
        outputMode = sdStereo;
    else
        outputMode = sdMono;



    /* Calculate one mixing element size: */
    if ( outputMode & sd16bit ) {
        mixElemSize = 2;
        	printf("16bit\n");
        }
    else {
        mixElemSize = 1;
    }
        
    if ( outputMode & sdStereo )
        mixElemSize <<= 1;

    /* Check correct mixing mode: */
    if ( outputMode & sdStereo )
        mixMode = dsmMixStereo;
    else
        mixMode = dsmMixMono;



    /* Allocate audio buffer */

	printf("elemsize: %i \n",mixElemSize);
	printf("mixrate: %i \n",mixRate);


    if ( outputMode & sd16bit ) {
		mixMode |= dsmMix16bit;
	    audioBufferSize = (mixRate * mixElemSize * mBufferLength / 1000);
	} else {
        mixMode |= dsmMix8bit;
	    audioBufferSize = (mixRate * mixElemSize * mBufferLength / 1000)/2;
	}
    if ( mOversampling )
	mixMode |= dsmMixInterpolation;

	
    /* Initialize Digital Sound Mixer: */
    if ( (error = dsmInit(mixRate, mixMode)) != OK )
        PASSERROR(ID_beosInit)

    /* Set update rate to 50Hz: */
    if ( (error = beosSetUpdRate(5000)) != OK )
        PASSERROR(ID_beosInit)

	

    if ( (error = memAlloc(audioBufferSize, (void**)&audioBuffer)) != OK )
        PASSERROR(ID_beosInit);



    /* Open output device using the format just set up: */


	if( outputMode & sd16bit )
		soco_mf.format = media_raw_audio_format::B_AUDIO_SHORT;
	else
		soco_mf.format 		= media_raw_audio_format::B_AUDIO_UCHAR;
	if( outputMode & sdStereo )
		soco_mf.channel_count 	= 2;	
	else
		soco_mf.channel_count	= 1;
	soco_mf.frame_rate		= mixRate;
	soco_mf.byte_order		= B_MEDIA_LITTLE_ENDIAN;
	soco_mf.buffer_size		= audioBufferSize;

	soco_bsbp = new BSoundPlayer( &soco_mf, "MIDAS", Be_Mix );

	soco_bsbp->SetVolume( 1.0 );
	soco_bsbp->Start();
//	soco_bsbp->SetHasData( true );


#ifdef DUMPBUFFER
    buff = fopen("buffer.raw", "wb");
#endif
    return OK;
}




/****************************************************************************\
*
* Function:     dsClose(void)
*
* Description:  Uninitializes DirectSound Sound Device
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING beosClose(void)
{
    
      int         error;

#ifdef DUMPBUFFER
    fclose(buff);
#endif


	soco_bsbp->Stop();
	
	printf("exit\n");
	delete soco_bsbp;
//	free( tmp_buffer );




    /* Uninitialize Digital Sound Mixer: */
    if ( (error = dsmClose()) != OK )
        PASSERROR(ID_beosClose)

    /* Deallocate audio buffer */
    if ( (error = memFree(audioBuffer)) != OK )
      PASSERROR(ID_beosClose);

//    close(audio_fd);

    return OK;
    
    
}




/****************************************************************************\
*
* Function:     int dsGetMode(unsigned *mode)
*
* Description:  Reads the current output mode
*
* Input:        unsigned *mode          pointer to output mode
*
* Returns:      MIDAS error code. Output mode is written to *mode.
*
\****************************************************************************/

int CALLING beosGetMode(unsigned *mode)
{
    *mode = outputMode;

    return OK;
}




/****************************************************************************\
*
* Function:     int dsSetUpdRate(unsigned updRate);
*
* Description:  Sets the channel value update rate (depends on song tempo)
*
* Input:        unsigned updRate        update rate in 100*Hz (eg. 50Hz
*                                       becomes 5000).
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING beosSetUpdRate(unsigned updRate)
{
    /* Calculate number of elements to mix between two updates: (even) */
    mixLeft = updateMix = ((unsigned) ((100L * (ulong) mixRate) /
        ((ulong) updRate)) + 1) & 0xFFFFFFFE;

    return OK;
}




/****************************************************************************\
*
* Function:     int dsStartPlay(void)
*
* Description:  Prepares for playing - we'll read the playing position here
*               to keep CPU usage about constant
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING beosStartPlay(void)
{

//	printf("start [%X]\n",soco_bsbp);
//	VC_PlayStart();

spaceLeft = audioBufferSize;
    return OK;
}




/****************************************************************************\
*
* Function:     int dsPlay(int *callMP);
*
* Description:  Plays the sound - mixes the correct amount of data with DSM
*               and copies it to the playback buffer.
*
* Input:        int *callMP             pointer to music player calling flag
*
* Returns:      MIDAS error code. If enough data was mixed for one updating
*               round and music player should be called, 1 is written to
*               *callMP, otherwise 0 is written there. Note that if music
*               player can be called, dsPlay() should be called again
*               with a new check for music playing to ensure the mixing buffer
*               gets filled with new data.
*
\****************************************************************************/

int CALLING beosPlay(int *callMP)
{
      int         error;

    do
    {
	
    /* Calculate number of mixing elements left: */
    numElems = audioBufferSize / mixElemSize;    
	
    if ( numElems > spaceLeft / mixElemSize )
        numElems = spaceLeft / mixElemSize;
    
    /* Check that we won't mix more data than there is to the next
       update: */
    if ( numElems > mixLeft )
        numElems = mixLeft;

    /* Decrease number of elements before next update: */
    mixLeft -= numElems;

	playing=1;
//	snooze(1000*200);
    

    /* Write the mixed data to the output device: */

//    write(audio_fd, audioBuffer, numElems * mixElemSize);

    spaceLeft -= numElems * mixElemSize;
    
    /* Check if the music player should be called: */
    if ( mixLeft == 0)
    {
        mixLeft = updateMix;
        *callMP = 1;
        return OK;
    }

    }while(spaceLeft);
	
	    
    /* No more data fits to the mixing buffers - just return without
       update: */
    *callMP = 0;

    return OK;
    
}






    /* WinWave Sound Device structure: */

SoundDevice     BeAudioMixer = {
    0,                                  /* tempoPoll = 0 */
    sdUseMixRate | sdUseOutputMode | sdUseDSM,  /* configBits */
    0,                                  /* port */
    0,                                  /* IRQ */
    0,                                  /* DMA */
    1,                                  /* cardType */
    1,                                  /* numCardTypes */
    sdMono | sdStereo | sd8bit | sd16bit,       /* modes */

    "BeOS Audio Mixer Sound Device " BEOSVERSTR,       /* name */
    &beosCardName,                                /* cardNames */
    0,                                          /* numPortAddresses */
    NULL,                                       /* portAddresses */

    &beosDetect,
    &beosInit,
    &beosClose,
    &dsmGetMixRate,
    &beosGetMode,
    &dsmOpenChannels,
    &dsmCloseChannels,
    &dsmClearChannels,
    &dsmMute,
    &dsmPause,
    &dsmSetMasterVolume,
    &dsmGetMasterVolume,
    &dsmSetAmplification,
    &dsmGetAmplification,
    &dsmPlaySound,
    &dsmReleaseSound,
    &dsmStopSound,
    &dsmSetRate,
    &dsmGetRate,
    &dsmSetVolume,
    &dsmGetVolume,
    &dsmSetSample,
    &dsmGetSample,
    &dsmSetPosition,
    &dsmGetPosition,
    &dsmGetDirection,
    &dsmSetPanning,
    &dsmGetPanning,
    &dsmMuteChannel,
    &dsmAddSample,
    &dsmRemoveSample,
    &beosSetUpdRate,
    &beosStartPlay,
    &beosPlay
#ifdef SUPPORTSTREAMS
    ,
    &dsmStartStream,
    &dsmStopStream,
    &dsmSetLoopCallback,
    &dsmSetStreamWritePosition,
    &dsmPauseStream
#endif
};


/*
 * $Log: dsound.c,v $
 * Revision 1.14  1997/07/31 14:30:04  pekangas
 * Added support for disabling DirectSound from the setup
 *
 * Revision 1.13  1997/07/31 10:56:40  pekangas
 * Renamed from MIDAS Sound System to MIDAS Digital Audio System
 *
 * Revision 1.12  1997/06/11 14:37:36  pekangas
 * Major cleanup for new mixing routines
 *
 * Revision 1.11  1997/05/30 18:26:45  pekangas
 * Fixed to work with new mixing routines
 *
 * Revision 1.10  1997/05/03 17:58:42  pekangas
 * Removed debug messages
 *
 * Revision 1.9  1997/05/03 17:54:46  pekangas
 * Now loads dsound.dll at run time if necessary - the DirectSound import
 * library is no longer needed
 *
 * Revision 1.8  1997/04/07 21:07:50  pekangas
 * Added the ability to pause/resume streams.
 * Added functions to query the number of stream bytes buffered
 *
 * Revision 1.7  1997/02/19 21:43:45  pekangas
 * Fixed retail build warnings
 *
 * Revision 1.6  1997/02/19 20:43:14  pekangas
 * Added Suspend() and Resume() -functions to Win32 sound devices
 *
 * Revision 1.5  1997/02/10 19:55:05  pekangas
 * Fixed to compile in debug mode
 *
 * Revision 1.4  1997/02/10 19:51:19  pekangas
 * Fixed removing debug messages (oops)
 *
 * Revision 1.3  1997/02/10 19:42:56  pekangas
 * Removed debug messages
 *
 * Revision 1.2  1997/02/08 18:25:53  pekangas
 * Fixed a Visual C warning
 *
 * Revision 1.1  1997/02/06 21:08:58  pekangas
 * Initial revision
 *
*/
